/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef JAC_AP_INNO3LIDARSERVICE_INNO3LIDARSERVICE_SKELETON_H
#define JAC_AP_INNO3LIDARSERVICE_INNO3LIDARSERVICE_SKELETON_H

#include "ara/com/internal/skeleton/skeleton_adapter.h"
#include "ara/com/internal/skeleton/event_adapter.h"
#include "ara/com/internal/skeleton/field_adapter.h"
#include "ara/com/internal/skeleton/method_adapter.h"
#include "ara/com/crc_verification.h"
#include "jac/ap/inno3lidarservice/inno3lidarservice_common.h"
#include <cstdint>

namespace jac {
namespace ap {
namespace inno3lidarservice {
namespace skeleton {
namespace events
{
    using LiDAR3PointCloudEvent = ara::com::internal::skeleton::event::EventAdapter<::LidarPointCloudNew>;
    static constexpr ara::com::internal::EntityId Inno3LiDARServiceLiDAR3PointCloudEventId = 8030U; //LiDAR3PointCloudEvent_event_hash
}

namespace methods
{
}

namespace fields
{
}

class Inno3LiDARServiceSkeleton {
private:
    std::unique_ptr<ara::com::internal::skeleton::SkeletonAdapter> skeletonAdapter;
    void ConstructSkeleton(const ara::com::MethodCallProcessingMode mode)
    {
        if (mode == ara::com::MethodCallProcessingMode::kEvent) {
            if (!(skeletonAdapter->SetMethodThreadNumber(skeletonAdapter->GetMethodThreadNumber(0U), 1024U))) {
#ifndef NOT_SUPPORT_EXCEPTIONS
                ara::core::ErrorCode errorcode(ara::com::ComErrc::kNetworkBindingFailure);
                throw ara::com::ComException(std::move(errorcode));
#else
                std::cerr << "Error: Not support exception, create skeleton failed!" << std::endl;
#endif
            }
        }
        const ara::core::Result<void> resultLiDAR3PointCloudEvent = LiDAR3PointCloudEvent.Initialize();
        ThrowError(resultLiDAR3PointCloudEvent);
    }

    Inno3LiDARServiceSkeleton& operator=(const Inno3LiDARServiceSkeleton&) = delete;

    static void ThrowError(const ara::core::Result<void>& result)
    {
        if (!(result.HasValue())) {
#ifndef NOT_SUPPORT_EXCEPTIONS
            ara::core::ErrorCode errorcode(result.Error());
            throw ara::com::ComException(std::move(errorcode));
#else
            std::cerr << "Error: Not support exception, create skeleton failed!" << std::endl;
#endif
        }
    }
public:
    class ConstructionToken {
    public:
        explicit ConstructionToken(const ara::com::InstanceIdentifier& instanceId,
            const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
            : processMode(mode),
              ptr(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::inno3lidarservice::Inno3LiDARService::ServiceIdentifier, instanceId, mode)),
              LiDAR3PointCloudEvent(ptr->GetSkeleton(), events::Inno3LiDARServiceLiDAR3PointCloudEventId, ::jac::ap::inno3lidarservice::Inno3LiDARService::ServiceIdentifier, instanceId){
        }

        explicit ConstructionToken(const ara::core::InstanceSpecifier& instanceSpec,
            const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
            : processMode(mode),
              ptr(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::inno3lidarservice::Inno3LiDARService::ServiceIdentifier, instanceSpec, mode)),
              LiDAR3PointCloudEvent(ptr->GetSkeleton(), events::Inno3LiDARServiceLiDAR3PointCloudEventId, ::jac::ap::inno3lidarservice::Inno3LiDARService::ServiceIdentifier, instanceSpec){
        }

        explicit ConstructionToken(const ara::com::InstanceIdentifierContainer instanceContainer,
            const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
            : processMode(mode),
              ptr(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::inno3lidarservice::Inno3LiDARService::ServiceIdentifier, instanceContainer, mode)),
              LiDAR3PointCloudEvent(ptr->GetSkeleton(), events::Inno3LiDARServiceLiDAR3PointCloudEventId, ::jac::ap::inno3lidarservice::Inno3LiDARService::ServiceIdentifier, instanceContainer){
        }

        ConstructionToken(ConstructionToken&& other)
            : processMode(std::move(other.processMode)),
              ptr(std::move(other.ptr)),
              LiDAR3PointCloudEvent(std::move(other.LiDAR3PointCloudEvent)){
        }
        ConstructionToken& operator=(ConstructionToken && other)
        {
            if (&other != this) {
                processMode = std::move(other.processMode);
                ptr = std::move(other.ptr);
                LiDAR3PointCloudEvent = std::move(other.LiDAR3PointCloudEvent);
            }
            return *this;
        }
        ConstructionToken(const ConstructionToken&) = delete;
        ConstructionToken& operator = (const ConstructionToken&) = delete;
        std::unique_ptr<ara::com::internal::skeleton::SkeletonAdapter> GetSkeletonAdapter()
        {
            return std::move(ptr);
        }
        events::LiDAR3PointCloudEvent GetLiDAR3PointCloudEvent()
        {
            return std::move(LiDAR3PointCloudEvent);
        }
        ara::core::Result<void> Initialize()
        {
            ara::core::Result<void> initResult;
            do {
                if (processMode == ara::com::MethodCallProcessingMode::kEvent) {
                    if(!ptr->SetMethodThreadNumber(ptr->GetMethodThreadNumber(0U), 1024U)) {
                        ara::core::ErrorCode errorCode(ara::com::ComErrc::kNetworkBindingFailure);
                        initResult.EmplaceError(errorCode);
                        break;
                    }
                }
                initResult = LiDAR3PointCloudEvent.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
            } while(false);

            return initResult;
        }
    private:
        ara::com::MethodCallProcessingMode processMode;
        std::unique_ptr<ara::com::internal::skeleton::SkeletonAdapter> ptr;
        events::LiDAR3PointCloudEvent LiDAR3PointCloudEvent;
    };
    explicit Inno3LiDARServiceSkeleton(const ara::com::InstanceIdentifier& instanceId,
                           const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
        : skeletonAdapter(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::inno3lidarservice::Inno3LiDARService::ServiceIdentifier, instanceId, mode)),
          LiDAR3PointCloudEvent(skeletonAdapter->GetSkeleton(), events::Inno3LiDARServiceLiDAR3PointCloudEventId, ::jac::ap::inno3lidarservice::Inno3LiDARService::ServiceIdentifier, instanceId){
        ConstructSkeleton(mode);
    }

    explicit Inno3LiDARServiceSkeleton(const ara::core::InstanceSpecifier& instanceSpec,
                           const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
        :skeletonAdapter(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::inno3lidarservice::Inno3LiDARService::ServiceIdentifier, instanceSpec, mode)),
          LiDAR3PointCloudEvent(skeletonAdapter->GetSkeleton(), events::Inno3LiDARServiceLiDAR3PointCloudEventId, ::jac::ap::inno3lidarservice::Inno3LiDARService::ServiceIdentifier, instanceSpec){
        ConstructSkeleton(mode);
    }

    explicit Inno3LiDARServiceSkeleton(const ara::com::InstanceIdentifierContainer instanceContainer,
                           const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
        :skeletonAdapter(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::inno3lidarservice::Inno3LiDARService::ServiceIdentifier, instanceContainer, mode)),
          LiDAR3PointCloudEvent(skeletonAdapter->GetSkeleton(), events::Inno3LiDARServiceLiDAR3PointCloudEventId, ::jac::ap::inno3lidarservice::Inno3LiDARService::ServiceIdentifier, instanceContainer){
        ConstructSkeleton(mode);
    }

    Inno3LiDARServiceSkeleton(const Inno3LiDARServiceSkeleton&) = delete;

    Inno3LiDARServiceSkeleton(Inno3LiDARServiceSkeleton&&) = default;
    Inno3LiDARServiceSkeleton& operator=(Inno3LiDARServiceSkeleton&&) = default;
    Inno3LiDARServiceSkeleton(ConstructionToken&& token) noexcept
        : skeletonAdapter(token.GetSkeletonAdapter()),
          LiDAR3PointCloudEvent(token.GetLiDAR3PointCloudEvent()){
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        ara::com::InstanceIdentifier instanceId,
        const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
    {
        ConstructionToken token(instanceId, mode);
        const auto initResult = token.Initialize();
        ara::core::Result<ConstructionToken> result(std::move(token));
        if (!initResult.HasValue()) {
            const ara::core::ErrorCode errorCode(initResult.Error());
            result.EmplaceError(errorCode);
        }
        return result;
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        ara::core::InstanceSpecifier instanceSpec,
        const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
    {
        ConstructionToken token(instanceSpec, mode);
        const auto initResult = token.Initialize();
        ara::core::Result<ConstructionToken> result(std::move(token));
        if (!initResult.HasValue()) {
            const ara::core::ErrorCode errorCode(initResult.Error());
            result.EmplaceError(errorCode);
        }
        return result;
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        ara::com::InstanceIdentifierContainer instanceIdContainer,
        const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
    {
        ConstructionToken token(instanceIdContainer, mode);
        const auto initResult = token.Initialize();
        ara::core::Result<ConstructionToken> result(std::move(token));
        if (!initResult.HasValue()) {
            const ara::core::ErrorCode errorCode(initResult.Error());
            result.EmplaceError(errorCode);
        }
        return result;
    }

    virtual ~Inno3LiDARServiceSkeleton()
    {
        StopOfferService();
    }

    void OfferService()
    {
        skeletonAdapter->RegisterE2EErrorHandler(&Inno3LiDARServiceSkeleton::E2EErrorHandler, *this);
        skeletonAdapter->OfferService();
    }
    void StopOfferService()
    {
        skeletonAdapter->StopOfferService();
    }
    ara::core::Future<bool> ProcessNextMethodCall()
    {
        return skeletonAdapter->ProcessNextMethodCall();
    }
    bool SetMethodThreadNumber(const std::uint16_t& number, const std::uint16_t& queueSize)
    {
        return skeletonAdapter->SetMethodThreadNumber(number, queueSize);
    }

    virtual void E2EErrorHandler(ara::com::e2e::E2EErrorCode, ara::com::e2e::DataID, ara::com::e2e::MessageCounter){}

    events::LiDAR3PointCloudEvent LiDAR3PointCloudEvent;
};
} // namespace skeleton
} // namespace inno3lidarservice
} // namespace ap
} // namespace jac

#endif // JAC_AP_INNO3LIDARSERVICE_INNO3LIDARSERVICE_SKELETON_H
